@@ -98,38 +98,45 @@ class HC(object) :
                         int(np.floor(gamma * self._N + 0.5)))
         return self._calculateHC(imin, imax)
     
-    def BJ(self, gamma=0.1) :
+    def berk_jones(self, gamma=1) :
         """
         Exact Berk-Jones statistic
 
+        According to Moscovich, Nadler, Spiegelman. (2013). 
+        On the exact Berk-Jones statistics and their p-value calculation
+
         Args:
         -----
         'gamma' : lower fraction of P-values to consider
 
         Returns:
         -------
-        -log(BJ) score, P-value attaining it
+        -log(BJ) score (large values are significant) 
+        (has a scaled chisquared distribution under the null)
+
         """
 
-        spv = self._pvals
         N = self._N
 
         if N == 0 :
             return np.nan, np.nan
+    
+        max_i = max(1, int(gamma * N))
+
+        spv = spv = self._pvals[:max_i]
+        ii = np.arange(1, max_i+1)
 
         bj = spv[0]
         p_th = spv[0]
-            
-        ii = np.arange(1, N + 1)
-        max_i = max(1, int(gamma * len(ii)))
-
-        if len(spv) >= 1 :
-            BJpv = beta.sf(spv, ii, N - ii + 1)[:max_i]
-            i_star = np.argmin(BJpv)
-            bj = BJpv[i_star]
-            p_th = spv[i_star]
         
-        return -np.log(bj), p_th
+        if len(spv) >= 1 :
+            BJpv = beta.cdf(spv, ii, N - ii + 1)
+            Mplus = np.min(BJpv)
+            Mminus = np.min(1 - BJpv)
+            bj = np.minimum(Mplus, Mminus)
+            
+        return -np.log(bj)
+
 
     def HCstar(self, gamma=0.2) :
         """sample-adjusted higher criticism score
@@ -416,7 +423,7 @@ def binom_var_test_df(c1, c2, sym=False, max_m=-1) :
     n1 = df_hist[(df_hist.n1 == 1) & (df_hist.n2 == 0)].N.values
     n2 = df_hist[(df_hist.n1 == 0) & (df_hist.n2 == 1)].N.values
     if len(n1) + len(n2) >= 2 :
-        df_hist.loc[df_hist.m == 1,'pval'] = binom_test_two_sided(n1, n1 + n2 , 1/2)
+        df_hist.loc[df_hist.m == 1, 'pval'] = binom_test_two_sided(n1, n1 + n2 , 1/2)[0]
 
     return df_hist
 
